@@ -6,6 +6,7 @@ import numpy as np
 from sklearn.ensemble import BaggingClassifier
 from sklearn.neighbors import KNeighborsClassifier
 from sklearn.utils import resample
+
 from multi_imbalance.resampling.soup import SOUP
 from multi_imbalance.utils.array_util import setdiff
 
@@ -15,7 +16,24 @@ def fit_clf(args):
 
 
 class SOUPBagging(BaggingClassifier):
+    """
+    Version of Bagging that applies SOUP in each classifier
+
+    Reference:
+    Lango, M., and Stefanowski, J. SOUP-Bagging: a new approach for multi-class
+    imbalanced data classification. PP-RAI ’19: Polskie Porozumienie na Rzecz Sztucznej
+    Inteligencji (2019).
+    """
+
     def __init__(self, classifier=None, maj_int_min=None, n_classifiers=5):
+        """
+        :param classifier:
+            Instance of classifier
+        :param maj_int_min:
+            dict {'maj': majority class labels, 'min': minority class labels}
+        :param n_classifiers:
+            number of classifiers
+        """
         super().__init__()
         self.classifiers, self.clf_weights = list(), list()
         self.maj_int_min = maj_int_min
@@ -37,7 +55,7 @@ class SOUPBagging(BaggingClassifier):
         out_of_bag = setdiff(np.hstack((X, y[:, np.newaxis])), np.hstack((x_sampled, y_sampled[:, np.newaxis])))
         x_out, y_out = out_of_bag[:, :-1], out_of_bag[:, -1].astype(int)
 
-        x_resampled, y_resampled = SOUP(maj_int_min=maj_int_min).fit_transform(x_sampled, y_sampled)
+        x_resampled, y_resampled = SOUP(maj_int_min=maj_int_min).fit_resample(x_sampled, y_sampled)
         clf.fit(x_resampled, y_resampled)
 
         result = clf.predict_proba(x_out)
@@ -51,12 +69,16 @@ class SOUPBagging(BaggingClassifier):
             print(f'Exc {Counter(y)} {Counter(y_out)} {result.shape} {expected_sum_prob.shape} {class_sum_prob.shape}')
         return clf, global_weights
 
-    def fit(self, X, y):
+    def fit(self, X, y, **kwargs):
         """
-
-        :param X: {array-like, sparse matrix} of shape = [n_samples, n_features] The training input samples.
-        :param y: array-like, shape = [n_samples]. The target values (class labels).
-        :return: self object
+        :param X:
+            array-like, sparse matrix of shape = [n_samples, n_features] The training input samples.
+        :param y:
+            array-like, shape = [n_samples]. The target values (class labels).
+        :param **kwargs:
+            dict (optional)
+        :return:
+            self object
         """
         self.classes = np.unique(y)
 
@@ -71,19 +93,24 @@ class SOUPBagging(BaggingClassifier):
 
         self.clf_weights = np.array(self.clf_weights)
 
-    def predict(self, X, strategy: str = 'average', maj_int_min: dict = None):
+    def predict(self, X, strategy: str = 'average'):
         """
         Predict class for X. The predicted class of an input sample is computed as the class with the highest
         sum of predicted probability.
 
-        :param X: {array-like, sparse matrix} of shape = [n_samples, n_features]. The training input samples.
+        :param X:
+            {array-like, sparse matrix} of shape = [n_samples, n_features]. The training input samples.
         :param strategy:
-            'average' - takes max from average values in prediction
-            'optimistic' - takes always best value of probability
-            'pessimistic' - takes always the worst value of probability
-            'mixed' - for minority classes takes optimistic strategy, and pessimistic for others. It requires maj_int_min
-        :param maj_int_min: dict. It keeps indices of minority classes under 'min' key.
-        :return: y : array of shape = [n_samples]. The predicted classes.
+            * 'average' :
+                takes max from average values in prediction
+            * 'optimistic' :
+                takes always best value of probability
+            * 'pessimistic' :
+                takes always the worst value of probability
+            * 'mixed' :
+                for minority classes takes optimistic strategy, and pessimistic for others. It requires maj_int_min
+        :return:
+            array of shape = [n_samples]. The predicted classes.
         """
         weights_sum = self.predict_proba(X)
         if strategy == 'average':
@@ -99,7 +126,7 @@ class SOUPBagging(BaggingClassifier):
 
             for i in range(n_classes):
                 two_dim_class_vector = weights_sum[:, :, i]  # [:,:,1] -> [classifiers x samples]
-                if i in maj_int_min['min']:
+                if i in self.maj_int_min['min']:
                     squeeze_with_strategy = np.max(two_dim_class_vector, axis=0)
                 else:
                     squeeze_with_strategy = np.min(two_dim_class_vector, axis=0)  # [1, n_samples, 1] -> [n_samples]
@@ -119,8 +146,10 @@ class SOUPBagging(BaggingClassifier):
         """
         Predict class probabilities for X.
 
-        :param X:{array-like, sparse matrix} of shape = [n_samples, n_features]. The training input samples.
-        :return: array of shape = [n_classifiers, n_samples, n_classes]. The class probabilities of the input samples.
+        :param X:
+            {array-like, sparse matrix} of shape = [n_samples, n_features]. The training input samples.
+        :return:
+            array of shape = [n_classifiers, n_samples, n_classes]. The class probabilities of the input samples.
         """
         n_samples = X.shape[0]
         n_classes = self.classes.shape[0]
